# NATools - Package with parallel corpora tools
# Copyright (C) 2002-2005  Alberto Simes
#
# This package is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

package NAT::Translator;

use strict;
use Data::Dumper;

use NAT::Client;

use NAT::Translator::method1;

use Time::HiRes qw(gettimeofday tv_interval );

sub new {
  my ($class) = @_;

  my $self = {};

  my $rank = NAT::Client::query_variable( "rank" );

  return bless $self, $class if $rank eq "rank";
  return undef;
}

# sub new {
#   my ($class,$dir,$slang) = @_;
#   my $self = {};
#   $self->{nat} = NAT::nat_open($dir);

#   return undef unless $self->{nat};
#   return undef unless $self->{nat}{rank};

#   # nat_open opens dictionaries on dic1 and dic2 but we need to look
#   # to $slang to know which one of them is the source language. For
#   # that we can compare with nat->{1|2}
#   #
#   # if they are in chunks or not, we have only one lexicon.
#   # Nat-open opens them, too, into lex1 and lex2.
#   #
#   # and pCorpus elements
#   #
#   if ($slang eq $self->{nat}{1}) {
#     # ok, get them directly
#     $self->{sDict} = $self->{nat}{dic1};
#     $self->{tDict} = $self->{nat}{dic2};
#     # and lexicons
#     $self->{sLex}  = $self->{nat}{lex1};
#     $self->{tLex}  = $self->{nat}{lex2};
#     # pCorpus Objects
#     my %ops = ();
#     if (ref($self->{nat}{crp1}) eq "ARRAY") {
#       my $nr = $self->{nat}{'nr-chunks'};
#       %ops = (chunks => $nr);
#       my $pc = NAT::PCorpus::new($self->{nat}{lex1file},
# 				 $self->{nat}{crp1_},
# 				 $self->{nat}{lex2file},
# 				 $self->{nat}{crp2_},
# 				 $self->{nat}{rank_},
# 				 %ops);
#       die "Error creating PCorpus element" unless $pc;
#       $self->{pCorpus} = [ $pc ];
#     } else {
#       my $pc = NAT::PCorpus::new($self->{nat}{lex1file},
# 				 $self->{nat}{crp1},
# 				 $self->{nat}{lex2file},
# 				 $self->{nat}{crp2},
# 				 $self->{nat}{rank},
# 				 %ops);
#       $self->{pCorpus} = [ $pc ];
#     }

#   } elsif ($slang eq $self->{nat}{2}) {
#     # swap them!
#     $self->{sDict} = $self->{nat}{dic2};
#     $self->{tDict} = $self->{nat}{dic1};
#     # and lexicons
#     $self->{sLex}  = $self->{nat}{lex2};
#     $self->{tLex}  = $self->{nat}{lex1};
#     # pCorpus Objects
#     my %ops = ();
#     if (ref($self->{nat}{crp1}) eq "ARRAY") {
#       my $nr = $self->{nat}{'nr-chunks'};
#       %ops = (chunks => $nr);
#       my $pc = NAT::PCorpus::new($self->{nat}{lex2file},
# 				 $self->{nat}{crp2_},
# 				 $self->{nat}{lex1file},
# 				 $self->{nat}{crp1_},
# 				 $self->{nat}{rank_},
# 				 %ops);
#       $self->{pCorpus} = [ $pc ];
#     } else {
#       my $pc = NAT::PCorpus::new($self->{nat}{lex2file},
# 				 $self->{nat}{crp2},
# 				 $self->{nat}{lex1file},
# 				 $self->{nat}{crp1},
# 				 $self->{nat}{rank},
# 				 %ops);
#       $self->{pCorpus} = [ $pc ];
#     }
#   } else {
#     return undef
#   }

#   return bless $self, $class;
# }


sub best_translation {
  my $self = shift;
  my $sstring = shift;

  my @ANS = $self->translate($sstring, @_);
  return ("",0) unless @ANS;

  # para cada ANS, pegar no melhor
  my @ANS = map {
    my ($tot,$A1) = @$_;
    my $k = (sort {$A1->{$b}<=>$A1->{$a}} keys %$A1)[0];
    [ $k, $A1->{$k} ]
  } @ANS;

  @ANS = sort {$b->[1]<=>$a->[1]} @ANS;
  return ("",0) unless defined $ANS[0][0];
  return @{$ANS[0]};
}


# sub translate {
#   my $self = shift;
#   my $sstring = lc(shift);
#   my %opts = @_;

#   my (%M1, %M2);
#   my (%A1, %A2);

#   my @search_words = split/\s+/,$sstring;
#   my $ssize = $#search_words + 1;
#   @search_words = map {$self->{sLex}->id_from_word($_)} @search_words;

#   my @ans;

#   # print "Starting corpus search...\n";
#   my $t0 = [gettimeofday];

#   my @ANS = ();

#   my @pcs = (ref($self->{pCorpus}) eq "ARRAY")?@{$self->{pCorpus}}:($self->{pCorpus});
#   for my $pc (@pcs) {
#     if (exists($opts{best})) {

#       # handle 'best => n'
#       # 1 - get the corpus occurrences
#       @ans = $pc->qsearch($sstring);
#       # 2 - if n == 0, then n == sample size
#       $opts{best} = scalar(@ans) unless $opts{best};
#       # 3 - sort by quality, and remove quality tag
#       @ans = map { [$_->[1],$_->[2]] } sort { $b->[0] <=> $a->[0] } @ans;
#       # 4 - get only the n best samples
#       @ans = @ans[0..$opts{best}-1] if $opts{best} < scalar(@ans);

#     } elsif (exists($opts{threshold})) {

#       # handle 'threshold => %'
#       @ans = $pc->qsearch($sstring);
#       @ans = map { [$_->[1],$_->[2]] } grep { $_->[0] >= $opts{threshold} } @ans;

#     } elsif (exists($opts{sample_size})) {

#       # handle 'sample_size => n'
#       @ans = $pc->search($sstring);
#       @ans = @ans[0..$opts{sample_size}-1] if $opts{sample_size} < scalar(@ans);

#     } elsif (exists($opts{smallest})) {

#       # handle 'smallest => n'
#       @ans = $pc->search($sstring);
#       $opts{smallest} = scalar(@ans) unless $opts{smallest};
#       @ans = sort { length($a->[0])+length($a->[1]) < length($b->[0])+length($b->[1])} @ans;
#       @ans = @ans[0..$opts{smallest}-1] if $opts{smallest} < scalar(@ans);

#     } else {
#       @ans = $pc->search($sstring);
#     }

#     #print "Done in ",tv_interval( $t0 )," seconds\n";
#     #print "Looking to occurrencies (window algorithm)\n";
#     $t0 = [gettimeofday];

#     for (@ans) {
#       my ($word1, $val1) = NAT::Translator::method1::method($self, \@search_words, $ssize, @$_);
#       $M1{$word1}{total}++;
#       $M1{$word1}{values}+=$val1;
#       # if ($opts{context} && $_->[1]=~m!$word1!) {
#       # $H1{$word1}{context} = [$`,$']
#       # }
#     }

#     #print "Done in ",tv_interval ( $t0 )," seconds\n";
#     %A1 = map { ($_ => $M1{$_}{values}/$M1{$_}{total} ) } grep {$M1{$_}{values} > 0.00001} keys %M1;

#     push @ANS, [scalar @ans, \%A1];
#   }
#   return @ANS;

# }


sub translate {
  my $self = shift;
  my $sstring = lc(shift);
  my %opts = @_;

  my (%M1, %M2);
  my (%A1, %A2);

  my @search_words = split/\s+/,$sstring;
  my $ssize = $#search_words + 1;

  my ($ans,@ans);

  # print "Starting corpus search...\n";
  my $t0 = [gettimeofday];

  my @ANS = ();

  $ans = NAT::Client::query({Count=>100},$sstring);
  @ans = @$ans;

  if (exists($opts{best})) {

    # handle 'best => n'

    # 2 - if n == 0, then n == sample size
    $opts{best} = scalar(@ans) unless $opts{best};

    # 3 - sort by quality, and remove quality tag
    @ans = map { [$_->[1],$_->[2]] } sort { $b->[0] <=> $a->[0] } @ans;

    # 4 - get only the n best samples
    @ans = @ans[0..$opts{best}-1] if $opts{best} < scalar(@ans);

  } elsif (exists($opts{threshold})) {

    # handle 'threshold => %'
    @ans = map { [$_->[1],$_->[2]] } grep { $_->[0] >= $opts{threshold} } @ans;

  } elsif (exists($opts{sample_size})) {

    # handle 'sample_size => n'
    @ans = @ans[0..$opts{sample_size}-1] if $opts{sample_size} < scalar(@ans);

  } elsif (exists($opts{smallest})) {

    # handle 'smallest => n'
    $opts{smallest} = scalar(@ans) unless $opts{smallest};
    @ans = sort { length($a->[0])+length($a->[1]) < length($b->[0])+length($b->[1])} @ans;
    @ans = @ans[0..$opts{smallest}-1] if $opts{smallest} < scalar(@ans);

  }

  #print "Done in ",tv_interval( $t0 )," seconds\n";
  #print "Looking to occurrencies (window algorithm)\n";
  $t0 = [gettimeofday];

  for (@ans) {
    my ($word1, $val1) = NAT::Translator::method1::method($self, \@search_words, $ssize, @$_);
    $M1{$word1}{total}++;
    $M1{$word1}{values}+=$val1;

  }

  #print "Done in ",tv_interval ( $t0 )," seconds\n";
  %A1 = map { ($_ => $M1{$_}{values}/$M1{$_}{total} ) } grep {$M1{$_}{values} > 0.00001} keys %M1;


  push @ANS, [scalar @ans, \%A1];

  return @ANS;

}



1;
__END__

=head1 NAME

NAT - Perl extension to perform 'translations' using NATools;

=head1 SYNOPSIS



=head1 DESCRIPTION



=head1 SEE ALSO

See perl(1) and NATools documentation.

=head1 AUTHOR

Alberto Manuel Brandao Simoes, E<lt>albie@alfarrabio.di.uminho.ptE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2002-2003 by NATURA Project
http://natura.di.uminho.pt

This library is free software; you can redistribute it and/or modify
it under the GNU General Public License 2, which you should find on
parent directory. Distribution of this module should be done including
all NATools package, with respective copyright notice.

=cut
