# NATools - Package with parallel corpora tools
# Copyright (C) 2002-2004  Alberto Simes
#
# This package is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

package NAT::Translator::method1;
use strict;
use Data::Dumper;



sub method {
  my ($self, $search_string, $size, $source, $target) = @_;

  return process($self, $size, $search_string, $target);
}

sub process {
  my ($self, $size, $search_string, $target) = @_;

  my @target = split/\s+/, $target;
  my $tsize = $#target + 1;
  my @target_ids = @target;

  my $margin = 1;
  my $low = $size-$margin;
  $low = 1 if $low < 1;

  my %data;
  for my $wsize ($low..$size+$margin) {
    my ($best_string, $best_value) = get_best_window($self,$size,$wsize,$search_string,$tsize,\@target_ids);
    $best_string =~ m!(\d+) (\d+)!;
    $best_string = join(" ", @target[$1..$2]);
    $data{$best_string} = $best_value;

  }
  my $best_str = (sort {$data{$b}<=>$data{$a} || length($a) > length($b)} keys %data)[0];
  my $best_val = $data{$best_str};

  return ($best_str,$best_val);

}

sub get_best_window {
  my ($self, $sss, $window_size, $search_string, $tsize, $target) = @_;
  my %answers = ();
  for my $i (0..$tsize-$window_size) {
    my @target_ids = @{$target}[$i..$i+$window_size-1];
    my $tstr = "$i ".($i+$window_size-1);
    $answers{$tstr} = full_sentence_similarity({size => $sss},$search_string, \@target_ids);
  }
  my $best_string = (sort {$answers{$b}<=>$answers{$a}} keys %answers)[0];
  my $best_value = $answers{$best_string};
  return ($best_string, $best_value);
}


sub full_sentence_similarity {
  return -1 unless ref $_[0];
  my $conf = shift;
  my $s1 = shift; # this must be a reference to an array
  my $s2 = shift; # same


  return -1 if not $s1 or not $s2 or not ref $s1 or not ref $s2;

  my $v1 = c_sentence_similarity( +{ %{$conf},
				     dir => "~>"},
				  $s1, $s2);
  my $v2 = c_sentence_similarity( +{ %{$conf},
				     dir => "<~"},
				  $s2, $s1);

  return ($v1 + $v2)/2;

}

sub c_sentence_similarity{
  return -1 unless ref $_[0];
  my $conf = shift;
  my ($s1,$s2) = @_;

  return -1 if not $s1 or not $s2;

  my $val = 0;

  # words for the source sentence
  my @wrds1 = @$s1;


  # number of words to be checked
  my $s1size = scalar(@wrds1);
  $s1size = $conf->{size} if exists($conf->{size});

  # create an associative array for quick test of words existence
  my %w2;
  @w2{@$s2} = @$s2;

  # for each word on the first sentence, get the best word on the target array.
  for my $w (@wrds1) {

    my $t = NAT::Client::query_dictionary({Direction => $conf->{dir}},
					  $w);

    if ($t) {
      my %trans = %{$t->[1]};
      for my $x (keys %trans) {
	if (exists($w2{$x})) {
	  $val += (1/$s1size) * ($trans{$x} || 0);
	  last;
	}
      }
    }
  }

  return $val;


}



1;
